<?php
/*
Plugin Name: LinkWorth
Plugin URI: http://www.linkworth.com
Description: LinkWorth Easy Link Syndication
Author: Aaron Harun
Version: 2.3.2
Author URI: http://ajaxedwp.com/

Content copyrighted and commisioned by: http://www.linkworth.com
Author: Aaron Harun, http://ajaxedwp.com

*/

// FULL ERROR CHECKING
// error_reporting(E_ALL);

global $lw_linkads; //Plugins are not initialized in global space, so we must define globals as such.
$lw_linkads = array();

//

//



class LinkWorth {

	/**********************************
	* Intitialize the plugin. Define variables, add actions,
	* optionally install the plugin.
	* Also, load LW data.
	***********************************/

	function init(){
	global $lw_linkads,$wp_rewrite,$lw_options;
		$lw_vercurrent = '2.3';

		add_action('admin_menu', array(&$this,'admin_menu_item'));
		add_action('wp_footer', array(&$this,'debug')); //Add debugging information.

		if(!is_array(get_option('lw_options'))){ // If the options array doesn't exist, this is the first run.
			$this->install();
		}

		$lw_options = get_option('lw_options');

		$this->set_constants();

		$this->get_ads();

		if(!$lw_options['billboard_base'])
			$lw_options['billboard_base'] = 'linkworth';

		if(is_array($lw_linkads)){ //If we have ads.

			$this->setup_filters_and_actions();

		}else{
			$this->lw_linkintxt_dbcleanup(); //Delete any old ads.
			$this->debug('No Linkworth ads found.'); //Add debug information.
			$lw_linkads = array();
		}
		
// 		 echo '<pre>';
// 		 print_r($lw_linkads);
// 		 echo '</pre>';

		$this->billboards_update(); //Update the Billboard ads. These are used with re-writes, so it is best to have it all ready.
		$this->links_update(); //Update the old links
		$this->linkintxts_update(); //Update the old links


		if($_GET['silent'] == 'no') //Disables silent running when specified.
			$lw_options['silent'] = 0;
	}

	/**********************************
	* Define the constants if they are not already set.
	* Define the options if they are not already set.
	***********************************/

	function set_constants(){
	global $lw_options;
		if(!defined('LW_WEBSITE_ID') || !defined('LW_HASH')){//If the constants are not defined, define them.
			if($lw_options['website_id'] && $lw_options['website_hash']){
				define('LW_WEBSITE_ID', $lw_options['website_id']);
				define('LW_HASH', $lw_options[website_hash]);
			}else{
				return; //If neither the options or the constants are defined, escape.
			}
		}else{
			if(!$lw_options['website_id'] || !$lw_options['website_hash']){//If the options are not defined, define them.
				$lw_options['website_id'] = LW_WEBSITE_ID;
				$lw_options['website_hash'] = LW_HASH;
			}
		}


	}

	/**********************************
	* Check how old the cache is, if it isn't old move on.
	* If there isn't a cache or it is expired, 
	* Download the new ads and parse them.
	***********************************/
	function get_ads(){
	global $lw_linkads;
		$lw_linkads = get_option('lw_cache');
		$last = get_option('lw_cache_time');
		if(!is_array($lw_linkads) || (time() - $last) > 3600){ 
			ini_set('default_socket_timeout', 10);

			$lw_advstring = @file_get_contents('http://www.linkworth.com/act/partner/code/plugin_feed.php?web_id=' . LW_WEBSITE_ID . '&hash=' . LW_HASH . '&format=xml');

			if($lw_advstring){
				include_once('LinkWorth_parser.php');

				$xml_parser = & new LW_XML();
				$lw_linkads = $xml_parser->parse($lw_advstring);
				$lw_linkads = $lw_linkads[linkworth][ads]; // Resync array to start with the ad blocks;
													// $lw_linkads[linkbb] etc.
				
				update_option('lw_cache',$lw_linkads);
				update_option('lw_cache_time',time());
			}else{

				update_option('lw_cache_time',time()-3000); //Delay the update for 10 minutes incase of server issues.

			}
		}
// echo $lw_advstring;
// echo '<pre>';
// print_r($lw_linkads);
// echo '</pre>';
	}


	/**********************************
	* Just add the filters and actions.
	***********************************/

	function setup_filters_and_actions(){

			//Automatic filter
			add_filter('the_content', array(&$this,'after_content'),12);	//Add billboard ads after content.
			//Manual theme editor
			add_action('lw_after_content', array(&$this,'lw_after_content')); //Add "type 1" ads

			add_filter('the_content', array(&$this,'linkwords_filter'),11); 	// Wraps post content in div tags.
			add_filter('the_content', array(&$this,'lw_contentreplace')); 	// LinkInTxt Filter on post content
	
			//Automatic filter
			add_filter('loop_end', array(&$this,'after_loop')); //Add "type 1" ads
			//Manual theme edit
			add_action('lw_after_loop', array(&$this,'lw_after_loop')); //Add "type 1" ads
			
			add_action('wp_footer', array(&$this,'footer')); //Add the normal footer.

			add_filter('query_vars', array($this,'addQueryVar')); //Billboards
			add_action('parse_query', array($this,'Query')); //Billboards
			add_action('get_bookmarks', array(&$this,'filter_bookmarks')); //Remove LW links so they are not displayed twice.
	}

	/**********************************
	* Install the plugin.
	***********************************/
	function install(){	
		global $wp_rewrite;
		if(get_option('lw_timeout')){ //this is an upgrade
			$options = $this->upgrade(); 
		}else{ //This is a new install
			// Store an array of values into the database.
			$options['lw_timeout'] = 5;
			$options['lw_linkcolor'] = 1;
			$options['lw_version'] = '1.0b7';
		}
		$wp_rewrite->flush_rules(); //Flush the rewrite rules.
		add_option('lw_options',$options,'The main array of options for LinkWorth');
		add_option('lw_widget');
		add_option('lw_widget');
		add_option('lw_links', '', 'Temporary home for linkworth links so we can compare quickly.');
		add_option('lw_cache', '', 'Cache of the lw ads in pretty serialized form.');
		add_option('lw_cache_time', '', 'Time the cache was last checked.');
	}

	/**********************************
	* Move the user's options over seamlessly
	***********************************/
	function upgrade(){

		$this->lw_linkintxt_dbcleanup();
		$lw_timeout = get_option('lw_timeout');
		$lw_sidebar = get_option('lw_sidebar');
		$lw_linktype = get_option('lw_linktype');
		$lw_linkcatid = get_option('lw_linkcatid');
		$lw_linkcolor = get_option('lw_linkcolor');
		$lw_cssmod = get_option('lw_cssmod');

		delete_option('lw_timeout');
		delete_option('lw_sidebar');
		delete_option('lw_linktype');
		delete_option('lw_linkcatid');
		delete_option('lw_linkcolor');
		delete_option('lw_cssmod');

	$options = compact($lw_cssmod,$lw_linkcolor,$lw_linkcatid,$lw_linktype,$lw_sidebar,$lw_timeout);

	return $options;
	}

	// ---------------------------------------------------------------------------------------
	// ---------------------------------- BILLBOARD Functions --------------------------------
	// ---------------------------------------------------------------------------------------

	/**********************************
	* If this is a billboard page, function outputs 
	* the billboard page and stops the rest of WP.
	***********************************/
	function billboards(){

		$billboard = get_query_var('linkworth');
		$lw_billboards = get_option('lw_billboards'); //Get the billboards array.

		if($billboard && $lw_billboards[$billboard]){ //CHeck if the requested billboard exists.
			echo @file_get_contents('http://www.linkworth.com/act/partner/code/linkbb.php?bb_id='.$lw_billboards[$billboard].'&web_id='.LW_WEBSITE_ID."&hash=".LW_HASH);
			die();
		}
	}

	/**********************************
	* Add the mod_rewrite rules to WordPress.
	* So we don't have to create a file.
	***********************************/

	function rewrite($wp_rewrite) {
	global $lw_options;

		$feed_rules = array($lw_options[billboard_base].'/(.+)?' => 'index.php?linkworth='. $wp_rewrite->preg_index(1));

		$wp_rewrite->rules = $feed_rules + $wp_rewrite->rules;
	}


	function addQueryVar($wpvar_array) {
	global $lw_options;

		$wpvar_array[] = 'linkworth';

		return($wpvar_array);
	}

	/**********************************
	* Before we do eanything else
	* Stop and check if this is a "billboard"
	***********************************/

	function Query() {
	global $wp_rewrite;
		if (get_query_var('linkworth')) {
			$this->billboards();
		}
	}

	/**********************************
	* Loop through the ads and update the billboard option with any new ones.
	***********************************/

	function billboards_update(){
	global $lw_linkads;
		$lw_billboards = array();
		$billboards = $lw_linkads['linkbb']['ad'];

		if(is_array($billboards)){
			foreach ($billboards as $billboard)	{
	
				$lw_billboards[$billboard['pagename']] = $billboard['bbid'];
		
			} // WHILE BILLBOARD CREATION LOOP ENDS
		}
		update_option('lw_billboards',$lw_billboards);
	}

	/**********************************
	* Loop through the ads and check for new links.
	* If there are new links, trigger an update.
	* It loops through the ads, if there is a new ad, it adds it to the array.
	* If there is an old ad that is still valid, it removes it.
	* If there is an old ad that is no longer valid it will not be "unset" and a simple count will reveal it.
	***********************************/

	function links_update(){
	global $lw_linkads;
		$lw_links = get_option('lw_links'); //Old links
		$lw_links_new = array(); //New links

		if(isset($lw_linkads))
			reset($lw_linkads);

		ob_start(); echo 'Saved Links:'; print_r($lw_links); $ads = ob_get_contents(); ob_end_clean(); $this->debug($ads);

		if(count($lw_linkads[linkads]) + count($lw_linkads[linkbb]) > 0){

			foreach($lw_linkads as $ad_type => $ads){


				if($ad_type != 'linkads' && $ad_type != 'linkbb') continue;

				if(!is_array($ads['ad'][0])){
					$ad = $ads['ad'];
					$ads['ad'] = null;
					$ads['ad'][0] = $ad;
				}

				foreach($ads['ad'] as $ad){ //Loop through all the ads of the same type.

					if($ad['location'] != "1" && $ad['location'] != "3") 
						continue;

					$url = ($ad_type == 'linkads')? $ad['url'] : $ad['pagename'];
					$anchor = $ad['anchor'];
					if($lw_links[$url] != $anchor){ //If this ad is not in our array, queue for update.
						$update = 1;
					}else{
						unset($lw_links[$url]); //Remove valid entry.
					}

					$lw_links_new[$url] = $anchor;
				} 
			}

		}else{
			$this->delete_sidebar_links();
		}

		ob_start(); echo 'Unmatched links:'; print_r($lw_links);  echo 'New Links:'; print_r($lw_links_new); $ads = ob_get_contents(); ob_end_clean(); $this->debug($ads);

		if($update || count($lw_links) > 0)// If there are old invalid ads or new ads, update the sidebar.
			$this->insert_sidebar_links();

		update_option('lw_links',$lw_links_new);
	}

	/**********************************
	* Loop through the ads and check for new linkintxts.
	* If there are new links, add them into an aray.
	***********************************/
	function linkintxts_update(){
	global $lw_linkads,$lw_options;
		$linkintxs = array(); //New links
		$links = $lw_linkads['linkintxt']['ad'];

		if(isset($lw_linkads))
			reset($lw_linkads);

		if(count($links) > 0){
			foreach ($links as $link){
				unset($add_word, $rep_word,$lw_dispurl);

				$add_word = $link['anchor'];
				$rep_word = "<a href=\"http://".$link['url']."\">".$link['anchor']."</a>";
				$lw_dispurl = $link['webpageurl'];

				if($add_word != "" ) {
					$linkintxs[] = array('find_word' => $add_word, 'replace_word' => $rep_word, 'disp_url' => $lw_dispurl);
				}

			}
			update_option('lw_linkintxs',$linkintxs);
		}else{
			$this->delete_sidebar_links();
		}

		ob_start(); echo 'LinkinTxts:'; print_r($linkintxs); $ads = ob_get_contents(); ob_end_clean(); $this->debug($ads);

		update_option('lw_linkintxs',$linkintxs);
	}

	// ---------------------------------------------------------------------------------------
	// ---------------------------------- Various utility Functions --------------------------------
	// ---------------------------------------------------------------------------------------

	/**********************************
	* Wraps the post content in div.
	***********************************/

	function linkwords_filter($content) {
	$lw_options = get_option('lw_options');
		if ($lw_options['lw_cssmod']) {
			$content = "<div id=\"lw_context_ads\">".$content."</div>\n";
			return $content;
		}
		return $content;
	}

	/**********************************
	* Filter all links that are in the LW category.
	* So they aren't displayed twice.
	* Only done if is_lw_widget is not set.
	***********************************/
	function filter_bookmarks($links){
	global $is_lw_widget;

		if($is_lw_widget){
			$bookmarks = $links;
		}else{
			$bookmarks = array();
			$lw_linkcatid = get_option('lw_linkcatid'); // Get the stored link category ID.
			foreach($links as $link){
				if($link->term_id != $lw_linkcatid)
					$bookmarks[] =$link;
			}
		}

	return $bookmarks;
	}

	/**********************************
	* Performs various tests on the theme.
	***********************************/

	function testtheme($test=''){
	static $site_html;
	
		if(!$site_html){ // Load the website only once per test run.
			$site_html = file_get_contents(get_option('siteurl').'?silent=no');
		}
	
		if($test == 'footer'){
			if(strpos($site_html,'<!-- LINKWORDS FOOTER CODE STARTS -->') !== false)
				return true;
		}

		if($test == 'loopexists'){
			if(strpos($site_html,'<!--linkworth loop ads start-->') !== false)
				return true;
		}

		if($test == 'loop'){
			$loop_count = preg_match_all('@\<\!--linkworth loop ads start--\>@',$site_html,$matches);
			if($matches[0][0])
				return count($loop_count);
			return 0;
		}
	
	return false; //No $test or all tests false.
	}

	/**********************************
	*
	***********************************/

	function cloudcss() {
	
		global $lw_options;
	
		$lw_cssfontsize = (rand()%11)+12;
		$lw_cssfontcolor = (rand()%3)+1;
	
		$options = get_option('lw_options');
		extract($options);

		if ($lw_linkcolor == 1) {
			// AQUAMARINE
			if ($lw_cssfontcolor == 1) { $lw_cssfontcolor = "#00aeff"; }
			elseif ($lw_cssfontcolor == 2) { $lw_cssfontcolor = "#594c9b"; }
			elseif ($lw_cssfontcolor == 3) { $lw_cssfontcolor = "#008c2e"; }
		} elseif ($lw_linkcolor == 2) {
			// FOREST
			if ($lw_cssfontcolor == 1) { $lw_cssfontcolor = "#798975"; }
			elseif ($lw_cssfontcolor == 2) { $lw_cssfontcolor = "#81ff60"; }
			elseif ($lw_cssfontcolor == 3) { $lw_cssfontcolor = "#2c8b14"; }
		} elseif ($lw_linkcolor == 3) {
			// WINTER
			if ($lw_cssfontcolor == 1) { $lw_cssfontcolor = "#737b87"; }
			elseif ($lw_cssfontcolor == 2) { $lw_cssfontcolor = "#77afff"; }
			elseif ($lw_cssfontcolor == 3) { $lw_cssfontcolor = "#14438b"; }
		} elseif ($lw_linkcolor == 4) {
			// SUMMER
			if ($lw_cssfontcolor == 1) { $lw_cssfontcolor = "#877673"; }
			elseif ($lw_cssfontcolor == 2) { $lw_cssfontcolor = "#ff7160"; }
			elseif ($lw_cssfontcolor == 3) { $lw_cssfontcolor = "#8b2014"; }
		} elseif ($lw_linkcolor == 5) {// FRUITY
			if ($lw_cssfontcolor == 1) { $lw_cssfontcolor = "#912dae"; }
			elseif ($lw_cssfontcolor == 2) { $lw_cssfontcolor = "#53a244"; }
			elseif ($lw_cssfontcolor == 3) { $lw_cssfontcolor = "#df723f"; }
		} elseif ($lw_linkcolor == 6) {// FRUITY
			if ($lw_cssfontcolor == 1) { $lw_cssfontcolor = "#ffff40"; }
			elseif ($lw_cssfontcolor == 2) { $lw_cssfontcolor = "#c0c0ff"; }
			elseif ($lw_cssfontcolor == 3) { $lw_cssfontcolor = "#ff80ff"; }
		} elseif ($lw_linkcolor == 7) {// FRUITY
			if ($lw_cssfontcolor == 1) { $lw_cssfontcolor = "#606060"; }
			elseif ($lw_cssfontcolor == 2) { $lw_cssfontcolor = "#a8a8a8"; }
			elseif ($lw_cssfontcolor == 3) { $lw_cssfontcolor = "#e2e2e2"; }
		} else {
			// ALL LINKS ARE RED! SOMETHING WENT WRONG ABOVE!
			if ($lw_cssfontcolor == 1) { $lw_cssfontcolor = "#ff0000"; }
			elseif ($lw_cssfontcolor == 2) { $lw_cssfontcolor = "#ff0000"; }
			elseif ($lw_cssfontcolor == 3) { $lw_cssfontcolor = "#ff0000"; }
		}

		$lw_options['csscolor'] = $lw_cssfontcolor;
		$lw_options['csssize'] = $lw_cssfontsize;
	
	}

	/**********************************
	* Adds footer to WP.
	***********************************/

	function footer() {
		global $lw_options;
		if ($lw_options['lw_cssmod']) {

			if(!$lw_options['silent'])
			  echo "\n<!-- LINKWORDS FOOTER CODE STARTS -->\n";

			echo "\n<style type='text/css'>\n.lw_cad_link:link {\n	text-decoration: underline;\n	border-bottom: 1px;\n}\n\n.lw_cad_link:visited {\n	text-decoration: underline;\n	border-bottom: 1px;\n}\n\n.lw_cad_link:hover {\n	text-decoration: underline;\n	border-bottom: 2px;\n}\n</style>\n\n<script type=\"text/javascript\" src=\"http://www.linkworth.com/context-ads/context_ads.php?prt_website_id=". LW_WEBSITE_ID ."\"></script>\n";

			if(!$lw_options['silent'])
			  echo "\n<!-- LINKWORDS FOOTER CODE ENDS -->\n";
		}
	}


	/**********************************
	* Adds LW menu item to WP dashboard
	***********************************/

	// CODE FOR THE LINKWORTH TAB ON THE OPTIONS MENU
	function admin_menu_item() {
		if (function_exists('add_options_page')) {
			add_options_page('LinkWorth Ads Syndication', 'LinkWorth', 8, __FILE__, 'lw_configpage');
		}
	}

	/**********************************
	* Updates admin panel options from posted data
	***********************************/
	function update_options($postdata){

		while (list($name, $value) = each($postdata)) {
			if( get_magic_quotes_gpc() ) {
				$value = stripslashes($value);
			}
			$options[$name] =$value;
		}

		update_option('lw_options',$options);

	return $options;
	}

	// ---------------------------------------------------------------------------------------
	// -----------------Ads before loop and after content Filter Functions --------------------
	// ---------------------------------------------------------------------------------------

	/**********************************
	* Automatically appends text links after post content.
	* Returns $content and any $ads
	***********************************/

	function after_content($content){
		global $lw_options;
		if(is_single() && !is_admin() && !is_feed() && !$lw_options['nocontentads']){ //If this is a normal post or page, add ads.
			$content = $content.$this->lw_after_content('return');
		}

		return $content;
	}

	/**********************************
	* Returns or echos any new ads.
	* If $return is passed, this function 
	* was called by our filter.
	* Otherwsie, it was the do_action
	***********************************/

	function lw_after_content($return = 0){
		global $lw_linkads,$lw_options;
		$count = 0;
		if(is_single()){ //If this is a normal post or page, add ads.

			foreach($lw_linkads as $ad_type => $ads){ //Loop the different ad types.
				if($ad_type == 'linkintxt' || ($ad_type == 'linkbb' && $lw_options['lw_sidebar'])) continue;
				foreach($ads['ad'] as $ad){ //Loop through all the ads of the same type.

					//If this ad should display on all pages or single pages.
					if ($ad['location'] == 3  || $ad['location'] == 2) { 

						//Run the display function for this $ad_type (functions are named after the xml names.)
						$ad_html .= $this->$ad_type($ad,$count);
						$count ++;
					}
				}
			}
		}

		if($return)
			return $ad_html;

		echo $ad_html;
	}

	/**********************************
	* Automatically adds ads after the loop.
	***********************************/

	function after_loop($content){
	global $lw_linkads,$lw_options;
	static $loop=1; //The number of WordPress loops that have been called so far.

		if(!is_home())
			return;

		if(!$lw_options['silent'])
			echo '<!--linkworth loop ads start-->';

		$loop_number = $lw_options[loop_number];

		if(!$loop_number)
			$loop_number = 1;
		$count = 0;
		if($loop == $loop_number  && !$lw_options['noloopads']){

			$this->lw_after_loop();
		}

		$loop++;

	}

	/**********************************
	* Echo all ads that belong on the homepage.
	***********************************/

	function lw_after_loop(){
	global $lw_linkads,$lw_options;
	static $loop=1; //The number of WordPress loops that have been called so far.

		if(!is_home())
			return;

		foreach($lw_linkads as $ad_type => $ads){ //Loop the different ad types.

			//Ignore billboard ads and sidebar ads when the sidebar is enabled.
			//Always ignore linkintxt ads.
			if($ad_type == 'linkintxt' || (($ad_type == 'linkbb' || $ad_type == 'linkads') && $lw_options['lw_sidebar'])) continue;

			foreach($ads['ad'] as $ad){ //Loop through all the ads of the same type.

				//If this ad should display on all pages or the home page.
				if ($ad['location'] == 3  || $ad['location'] == 1) { 

					//Run the display function for this $ad_type (functions are named after the xml names.)
					$ad_html .= $this->$ad_type($ad,$count);
					$count++;
				}
			}
		}

		echo $ad_html;
	}

	// ---------------------------------------------------------------------------------------
	// ---------------------------------- Ad Display Functions -------------------------------
	// ---------------------------------------------------------------------------------------

	function linkbb($ad,$count=0){
	global $id,$lw_options;

		if(!is_array($ad))
			return $ads;

		$this->link_options($count, $ad);

		$ads =  $lw_options['lw_seperator'].'<a href="'.get_option('siteurl').$lw_options[billboard_base].'/'.$ad[pagename].'"'.$lw_options['lw_style'].$lw_options['rel'].'>'.$ad[anchor].'</a>';

		return $ads;

	}

	function linkads($ad, $count = 0){
	global $lw_options;

		if(!is_array($ad))
			return $ads;

		$this->link_options($count, $ad);

		$ads =  $lw_options['lw_seperator'].'<a href="http://'.$ad[url].'"'.$lw_options['lw_style'].$lw_options['rel'].'>'.$ad[anchor].'</a>';

		return $ads;
	}

	function linkmura($ad, $count = 0){
	global $lw_options;

		if(!is_array($ad))
			return $ads;

		$this->link_options($count, $ad);

		$lw_arraysize = count($ad[hyperlinks][hyperlink]);		// NUMBER OF ANCHOR TEXTS
		$lw_mura_rand = rand(0,$lw_arraysize-1);		// PICK A RANDOM ANCHOR TEXT
		$ad = $ad[hyperlinks][hyperlink][$lw_mura_rand];

		$ads =  $lw_options['lw_seperator'].'<a href="http://'.$ad[url].'"'.$lw_options['lw_style'].$lw_options['rel'].'>'.$ad[anchor].'</a>';
		return $ads;
	}

	function linksura($ad, $count = 0){
	global $lw_options;

		if(!is_array($ad))
			return $ads;

		$this->link_options($count, $ad);

		$lw_arraysize = count($ad[anchors][anchor]);		// NUMBER OF ANCHOR TEXTS
		$lw_sura_rand = rand(0,$lw_arraysize-1);		// PICK A RANDOM ANCHOR TEXT

		$ads =  $lw_options['lw_seperator'].'<a href="http://'.$ad[url].'"'.$lw_options['lw_style'].$lw_options['rel'].'>'.$ad[anchors][anchor][$lw_sura_rand].'</a>';
		return $ads;
	}

	/**********************************
	* Sets Style, rel and seperator for each link.
	* Uses global $lw_options array to temporarily
	* store information.
	***********************************/

	function link_options($count, $ad){
		global $lw_options;

			$lw_options['lw_style'] = '';
			if ($lw_options['lw_linkcolor']) {
				$this->cloudcss();
				$lw_options['lw_style'] = ' style="color:'.$lw_options['csscolor'].'; font-size: '.$lw_options['csssize'].'px;"';
			}
	
			$lw_options['rel'] = '';
			if($ad['nofollow'])
				$lw_options['rel'] = ' rel="nofollow"';
	
			// SORTING OUT WHAT KIND OF LINK SEPARATOR WE NEED TO PRINT, IF ANY
			if ($count == 0) {
				$lw_options['lw_seperator'] = '';
			} else {
				if ($lw_options[csscolor]) {
					$lw_options['lw_seperator'] = ' &nbsp; &nbsp; &nbsp;';
				} else {
					$lw_options['lw_seperator'] = ' - ';
				}
			}
	}
	// ---------------------------------------------------------------------------------------
	// ---------------------------------- LinkinText Functions -------------------------------
	// ---------------------------------------------------------------------------------------

	/**********************************
	*
	***********************************/

	function lw_linkintxt_dbcleanup() {
	
		// REMOVE OLD LINKINTXT ADVERTS
		global $table_prefix, $wpdb, $user_level;
		$table_name = $table_prefix . "linkintxt";
	
		$lw_cleandb = "DROP TABLE IF EXISTS ".$table_name;
		@$wpdb->query($lw_cleandb);
	
	}

	/**********************************
	*
	***********************************/

	function replace_ok($fwords, $rwords, $content) {
	
		$rcontent = "";
		$ncon = preg_split("/\b".$fwords[0]."\b/i", $content, 2);
		
		$f = array_shift($fwords);
		$r_lw = array_shift($rwords);
		
		$lw_oneword = 0;
		
		for($lw_i = 0; $lw_i < sizeof($ncon); $lw_i+=1)
		{
		
			$con = $ncon[$lw_i];
		
			if(sizeof($fwords)) {
				$con = $this->replace_ok($fwords, $rwords, $con);
			} 
		
			if($lw_i < sizeof($ncon)-1) {
				$rcontent .= $con.$r_lw;
			}
		
			else {
				$rcontent .= $con;
			}
		
		}
	
	return $rcontent;
	}

	/**********************************
	* ReplaceOne takes the input word and replaces it with the output.
	***********************************/

	function ReplaceOne($in, $out, $content){
		if (strpos($content, $in) !== false){
			$pos = strpos($content, $in);
			return substr($content, 0, $pos) . $out . substr($content, $pos+strlen($in));
		} else {
			return $content;
		}
	}

	/**********************************
	* $finds -- an array of words to look for and replace with.
	***********************************/

	function lw_contentreplace($content) {
	
		global $user_level, $post;
	
		$find_words = array();
		$rep_words = array();

		$finds = get_option('lw_linkintxs');

		if(count($finds) <= 1)
			return $content;

		$links = array();
		$tags = array();
	
		if (sizeof($finds)) {
	
			//strip all the existing links out first .
			$pattern = '/(<(?:[^<>]+(?:"[^"]*"|\\\'[^\']*\\\')?)+>)/';
			$html_array = preg_split ($pattern, $content, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
			$inlink = false;
			$content = "";
	
			for ($lw_i = 0; $lw_i < sizeof($html_array); $lw_i+=1) {
			
				$chunk = $html_array[$lw_i];
	
				if ((strpos($chunk, "<a") == 0 && strpos($chunk, "<a") !== false) || (strpos($chunk, "<A") == 0 && strpos($chunk, "<A") !== false)) {
					//strip out all three link parts and replace with a ***
					$links[] = $chunk.$html_array[++$lw_i].$html_array[++$lw_i];
					$content .= "!@#$%^&*()";
				} elseif ((strpos($chunk, "<") == 0 && strpos($chunk, "<") !== false)) { // If the first character is an <
					//this means it is a random tag, so strip it and hold it
					$tags[] = $chunk;
					$content .= "!@#T%^&*()";
				} else {
					$content .= $chunk;
				}
		
			}

			$lw_pageurl = $_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"];
			$lw_pageurl = preg_replace('@http(s)*:\/\/(www.)*@','',$lw_pageurl);
			$lw_pageurl = rtrim($lw_pageurl,'/');

			foreach($finds as $find) {
				$lw_dispurl = preg_replace('@http(s)*:\/\/(www.)*@','',$find['disp_url']);
				$lw_dispurl = rtrim($lw_dispurl,'/');
				$this->debug('LinkinTXT: Display URL: '.$lw_dispurl.'; Page URL: '.$lw_pageurl.'; Find:'.$find['find_word'].'; Replace With: '.$find['replace_word']);
				// BELOW LINE SET TO != FOR TESTING PURPOSES
				if ($lw_dispurl = $lw_pageurl) {
					$find_words[] = $find['find_word'];
					$repl_words[] = $find['replace_word'];
				}
			}
		
		}

		if (sizeof($find_words)) {
			$content = $this->replace_ok($find_words, $repl_words, $content);
		}

		foreach($tags as $tag) {
			$content = $this->ReplaceOne("!@#T%^&*()", $tag, $content);
		}
	
		foreach($links as $link) {
			$content = $this->ReplaceOne("!@#$%^&*()", $link, $content);
		}
	
	return $content;
	
	}

	/**********************************
	* insert_sidebar_links inserts billboard ads and Linkads
	***********************************/
	function insert_sidebar_links(){
	global $lw_linkads, $lw_options;

		
		reset ($lw_linkads); // Reset array pointer

		$this->delete_sidebar_links(1); //Remove outdated links.
		$lw_linkscatid = get_option('lw_linkcatid');

		if(!$lw_linkscatid || get_cat_name($lw_linkscatid) == ''){

			/*-------------------------------------
			* wp_insert_term accepts 3 arguments: 
			*	The first is the "term" name, 
			*	The second is the type. In this case "link_category."
			*	The third is an array of data with the same names as the database fields.
			* The function returns the ID of the insterted category, which we then store.
			-------------------------------------*/

			$lw_cat = array();
			
			$lw_cat[slug] = 'lwpartners';

			$lw_linkscatid = wp_insert_term('LinkWorth Partners', 'link_category', $lw_cat );
			$lw_linkscatid = $lw_linkscatid[term_id];
			update_option('lw_linkcatid', $lw_linkscatid);

		}
		$lw_path = get_option('siteurl');

		if(!is_array($lw_linkads))
			return;

		foreach($lw_linkads as $ad_type => $ads){

			if($ad_type != 'linkads' && $ad_type != 'linkbb') continue;

				if(!is_array($ads['ad'][0])){
					$ad = $ads['ad'];
					$ads['ad'] = null;
					$ads['ad'][0] = $ad;
				}

			foreach($ads['ad'] as $ad){ //Loop through all the ads of the same type.



				if ($ad_type == 'linkads') {
	
						// CLEAR ALL VARIABLES TO PREVENT ANY PROBLEMS
						unset ($linkdata);
	
						/**********************************
						* wp_insert_link accepts an array of data with the same names as the database fields.
						* Data is passed as an array which is then extracted and inserted into the database.
						***********************************/
	
						$linkdata = array();
	
						$linkdata['link_name'] = $ad['anchor'];//Link Name/ Anchor Text
						$linkdata['link_url'] = "http://".$ad['url'];//Link URL
						$linkdata['link_category'][] = $lw_linkscatid; //WordPress Category
	
						wp_insert_link($linkdata);  //Insert Link.	

				} elseif ($ad_type == 'linkbb') {	
						
						// LETS INSERT A LINK!

						if(get_option('permalink_structure') ==''){
							$lw_filename = $lw_path.'?linkworth='.$ad['pagename'];
						}else{
							$lw_filename = $lw_path.'/'.$lw_options[billboard_base].'/'.$ad['pagename'];
						}
	
						$linkdata = null;
						$linkdata = array();
	
						$linkdata['link_name'] = $ad['anchor'];//Link Name/ Anchor Text
						$linkdata['link_url'] = $lw_filename;//Link URL
						$linkdata['link_category'][] = $lw_linkscatid; //WordPress Category
	
						wp_insert_link($linkdata);  //Insert Link.
	
				} // CLOSING BILLBOARD ADS IF PORTION
			}
		}
	}

	/**********************************
	* insert_sidebar_links inserts billboard ads and Linkads
	***********************************/
	function delete_sidebar_links($only_links=0){
		$lw_linkcatid = get_option('lw_linkcatid'); // Get the stored link category ID.

		if(!function_exists('wp_delete_link'))
			@include_once(ABSPATH.'/wp-admin/includes/bookmark.php');

		if(!$lw_linkcatid)
			return;

		/**********************************
		* get_bookmarks accepts 1 array as an argument.
		* We only need the links for the default LW category so we can delete them.
		***********************************/
		remove_action('get_bookmarks', array(&$this,'filter_bookmarks'));
		$args = array();
		$args[category] = $lw_linkcatid; //Set up the array arguments.	
		$args[limit] = rand(999,9999); //Cache buster
		$links = get_bookmarks($args);

		if(!function_exists('wp_delete_link'))
			@include_once(ABSPATH.'/wp-admin/includes/bookmark.php');

		if(count($links) > 0){
			foreach($links as $link){
				wp_delete_link($link->link_id); //Delete the individual links.
			}
		}

		if(!$only_links){
			delete_option('lw_linkcatid');
			wp_delete_term($lw_linkcatid, 'link_category'); //Delete the LW category.
		}
	}

	/**********************************
	* Debugging information
	* Stores and prints an array.
	***********************************/
	function debug($message=''){
	static $messages = array();

		$ops = get_option('lw_options');

		if($message){

			$messages[] = $message;

		}elseif($ops[debug] == 1){

			foreach($messages as $debug){
  
				echo "\n\r".'<!-- LW Debug: '.$debug.' -->';

			}
		}


	}

	/**********************************
	* Deactivate the plugin. Deletes the old links etc.
	***********************************/
	function deactivate_hook(){
		$this->delete_sidebar_links();
	}

	/**********************************
	* Returns a "normal" description of the ad_location value.
	***********************************/
	function human_location($loc){

		if ($loc == 1) { $lw_humanloc = 'Home Page'; }
		elseif ($loc == 2) { $lw_humanloc = 'Sub Pages'; }
		elseif ($loc == 3) { $lw_humanloc = 'All Pages'; }
		else { $lw_humanloc =$ad['location']; }

		return $lw_humanloc;
	}

}// CLASS END



	// -------------------------------------------------------------------------------
	// ------------------------- Initialize the LW CLass -----------------------------
	// -------------------------------------------------------------------------------

$LinkWorth = new LinkWorth;
add_action('init', array(&$LinkWorth, 'init')); 	// Initiate LW. //Passed by reference for PHP4
register_deactivation_hook(__file__,array($LinkWorth,'deactivate_hook'));
add_filter('generate_rewrite_rules', array($LinkWorth,'rewrite')); //Billboards




	// ---------------------------------------------------------------------------------------
	// ---------------------------------- LW Widget -------------------------------
	// ---------------------------------------------------------------------------------------

function widget_linkw_init() {

	if ( !function_exists('register_sidebar_widget') )
		return;

	function widget_linkw($args) {
		global $is_lw_widget,$LinkWorth;
		extract($args);

		$options = get_option('lw_widget');
		$lw_linkcatid = get_option('lw_linkcatid');
		$title = $options[title];
		$is_lw_widget = 1;
		remove_action('get_bookmarks', array(&$LinkWorth,'filter_bookmarks'));
 		wp_list_bookmarks(array(
 			'title_before' => $before_title, 'title_after' => $after_title,
 			'category_before' => $before_widget, 'title_li' => $title, 'category_after' => $after_widget,
 			'show_images' => true, 'class' => 'linkcat widget', 'categorize' => 0, 'category'=>$lw_linkcatid
 		));	
		add_action('get_bookmarks', array(&$LinkWorth,'filter_bookmarks')); //Remove LW links so they are not displayed twice.
		$is_lw_widget=0;
	}

	function widget_linkw_control() {
		$options = get_option('lw_widget');

		if ( !is_array($options) ){
			$options = array('title'=>'LinkWorth');
			add_option('lw_widget', $options);
		}

		if ( $_POST['linkw-submit'] ) {
			$options['title'] = strip_tags(stripslashes($_POST['linkw-title']));
			update_option('lw_widget', $options);
		}

		$title = htmlspecialchars($options['title'], ENT_QUOTES);
		// Here is our little form segment. Notice that we don't need a
		// complete form. This will be embedded into the existing form.
		echo '<p style="text-align:right;"><label for="linkw-title">Title: <input style="width: 200px;" id="linkw-title" name="linkw-title" type="text" value="'.$title.'" /></label></p>';
		echo '<input type="hidden" id="linkw-submit" name="linkw-submit" value="1" />';
	}

	// This registers our widget so it appears with the other available
	// widgets and can be dragged and dropped into any active sidebars.
	register_sidebar_widget('LinkWorth Links', 'widget_linkw');

	// This registers our optional widget control form. Because of this
	// our widget will have a button that reveals a 300x100 pixel form.
	register_widget_control('LinkWorth Links', 'widget_linkw_control', 300, 615);
}
$lw_options = get_option('lw_options');
if($lw_options['lw_sidebarwidget']){//If widgets are selected.
	add_action('init', 'widget_linkw_init');
}






	// ---------------------------------------------------------------------------------------
	// ---------------------------------- LW ADMIN Panel -------------------------------
	// ---------------------------------------------------------------------------------------
/**********************************
* Called by the admin panel function to display advertisement information.
* In its own function to "keep it clean"
***********************************/
function lw_configpage() {
	
	include('LinkWorth_admin.php');
	
}




?>
